﻿-- Rift Add-On start-up framework
-- Design to sync with Character load assignments (self)
-- Written By Paul Snart

QuestFinder_Data = {}

local AddonInfo, QFind = ...
local _qf = {
	questList = {},
	questLog = {},
	Settings = {
		console = "general",
		ID = "v0000000000000000",
		silent = false,
	},
	Default = {
		ID = "v0000000000000000",
	},
	chatPrefix = '<font color="#FAFA09">[QF] <font color="#FFFFFF">',
}
QuestFinder_Settings = _qf.Settings

function _qf.VarLoad(handle, Addon)
	if Addon == AddonInfo.id then
		if QuestFinder_Data then
			if QuestFinder_Data.Cache then
				_qf.questList = QuestFinder_Data.Cache
			end
		else
			QuestFinder_Data = {}
		end
		_qf.Settings = QuestFinder_Settings
	end
end

function _qf.VarSave(handle, Addon)
	if Addon == AddonInfo.id then
		_qf.questList["q"] = nil
		QuestFinder_Data.Cache = _qf.questList
		QuestFinder_Settings = _qf.Settings
	end
end

function _qf:StoreQuests()
	local questList = Inspect.Quest.Complete()
	local TotalComplete = 0
	local nID
	if questList then
		for qID, data in pairs(questList) do
			TotalComplete = TotalComplete + 1
			if string.find(qID, "FFFFFFFF") then
				nID = "q"..string.sub(qID, 10, 17)
			else
				nID = string.sub(qID, 1, 9)
			end
			if string.len(nID) == 9 then
				if not self.questList[nID] then
					self.questList[nID] = true
				end
			end
		end
	end
end

function _qf:StoreQuestLog()
	self.questLog = Inspect.Quest.List()
end

function _qf.Gather(zDetails, uDetails, ID)

	local zObj
	if ID then
		zObj = QFind.Data[zDetails.id]
	else
		zObj = QFind.Data[zDetails.name]
	end
	
	local Complete = 0
	local qError = 0
	local count = 0
	local Total = zObj.Total
	local Carnage = zObj.Carnages	
	local CompleteName = {}
	local MissingName = {}
	local InLogName = {}
	local endTime = Inspect.Time.Real() + 0.02
	
	for Faction, fObject in pairs(zObj) do
		if type(fObject) == "table" then
			if Faction == "Both" or string.lower(Faction) == uDetails.alliance then
				for QuestID, qObject in pairs(fObject) do
					local attempts = 0
					local tempQuest = _qf.questList[QuestID]
					count = count + 1					
					repeat
						if type(_qf.questList[QuestID]) ~= "table" then
							tempQuest = Inspect.Quest.Detail(qObject.AddonId)
						end
						if tempQuest then
							if _qf.questList[QuestID] then
								Complete = Complete + 1
								CompleteName[tempQuest.name] = tempQuest
								_qf.questList[QuestID] = tempQuest
								if MissingName[tempQuest.name] then
									MissingName[tempQuest.name] = nil
									Total = Total - 1
								end
							else
								if CompleteName[tempQuest.name] then
									Total = Total - 1
								else
									if _qf.questLog[qObject.AddonId] then
										InLogName[tempQuest.name] = tempQuest
									else
										if not MissingName[tempQuest.name] then
											MissingName[tempQuest.name] = tempQuest
										else
											Total = Total - 1
										end
									end
								end
							end
						else
							coroutine.yield()
							endTime = Inspect.Time.Real() + 0.02
							attempts = attempts + 1
							if attempts > 50 then
								Total = Total - 1
								break
							end
						end
					until tempQuest
					if Inspect.Time.Real() > endTime then
						coroutine.yield()
						endTime = Inspect.Time.Real() + 0.02
					end
				end
			else
				for QuestID, qObject in pairs(fObject) do
					Total = Total - 1
				end
			end
			if Inspect.Time.Real() > endTime then
				coroutine.yield()
				endTime = Inspect.Time.Real() + 0.02
			end
		end
	end
	if _qf.DisplayComplete then
		for QuestName, qObject in pairs(CompleteName) do
			_qf.print('<font color="#00FF00">'..QuestName)
		end
		_qf.print("--")
		_qf.print(string.format(QFind.Locale.Found, Total, Carnage))
		_qf.print(string.format(QFind.Locale.Completed, Complete))
	else
		local Missing = 0
		local InLog = 0
		for QuestName, qObject in pairs(InLogName) do
			_qf.print(string.format(QFind.Locale.Log, QuestName))
			InLog = InLog + 1
		end
		_qf.print("--")
		for QuestName, qObject in pairs(MissingName) do
			_qf.print(string.format('<font color="#FFFF00">'..QFind.Locale.Missing, QuestName))
			if _qf.DisplayFull then
				local skip = false
				if qObject.summary then
					if qObject.objective then
						for i, Obj in ipairs(qObject.objective) do
							if Obj.indicator then
								for i2, Obj2 in ipairs(Obj.indicator) do
									if Obj2.x and Obj2.z then
										_qf.print(string.format("%s (%d, %d)", qObject.summary, Obj2.x, Obj2.z))
										_qf.print("--")
										skip = true
										break
									end
								end
							end
						end
					end
					if not skip then
						if qObject.summary then
							_qf.print(qObject.summary)
							_qf.print("--")
						end
					end
				end
			end
			Missing = Missing + 1
		end
		_qf.print(string.format(QFind.Locale.Found, Total, Carnage))
		if Total <= Complete then
			_qf.print(QFind.Locale.Finished)
		else
			_qf.print(string.format(QFind.Locale.Completed, Complete))
			_qf.print(string.format(QFind.Locale.Summary, Missing, InLog))
		end	
	end
end

function _qf.print(text)
	if type(text) ~= "string" then
		return
	end
	Command.Console.Display(_qf.Settings.ID, true, text, true)
end

function _qf.Search(handle, data)
	data = data or ""
	data = string.lower(data)
	if _qf.Gatherer then
		_qf.print(QFind.Locale.Patient)
	else
		if _qf.PlayerAvailable then
			local uDetails = Inspect.Unit.Detail("player")
			if uDetails then
				if uDetails.zone then
					local zDetails = Inspect.Zone.Detail(uDetails.zone)
					if zDetails then
						_qf.print(string.format(QFind.Locale.ZName, zDetails.name))
						local zObj = QFind.Data[zDetails.name]
						local ID = false
						if not zObj then
							zObj = QFind.Data[zDetails.id]
							if zObj then
								ID = true
							end
						end
						if zObj then
							if data == "full" then
								_qf.DisplayFull = true
								_qf.DisplayComplete = false
							elseif data == "done" then
								_qf.DisplayFull = false
								_qf.DisplayComplete = true
							else
								_qf.DisplayFull = false
								_qf.DisplayComplete = false
							end
							_qf:StoreQuests()
							_qf:StoreQuestLog()
							_qf.Gatherer = coroutine.create(_qf.Gather)
							assert(coroutine.resume(_qf.Gatherer, zDetails, uDetails, ID))
						else
							_qf.print(QFind.Locale.NoQuest)
						end
					else
						_qf.print(QFind.Locale.NoLoad)
					end
				else
					_qf.print(QFind.Locale.NoZone)
				end
			else
				_qf.print(QFind.Locale.NoPlayer)
			end
		else
			_qf.print(QFind.Locale.SlowRift)
		end
	end
end

function _qf.Tick()
	if _qf.PlayerAvailable then
		if _qf.Gatherer then
			if coroutine.status(_qf.Gatherer) == "dead" then
				_qf.Gatherer = nil
			else
				assert(coroutine.resume(_qf.Gatherer))
			end
		end
	end
end

function _qf.CheckPlayer(handle, units)
	if units[_qf.PlayerLoaded] then
		_qf.PlayerAvailable = true
		_qf.Tick()
	end
end

function _qf.FindConsole()
	local conList = Inspect.Console.List()
	if conList then
		if _qf.Settings.console == "general" then
			if conList[_qf.Settings.ID] then
				return
			else
				_qf.Settings.ID = _qf.Default.ID
				_qf.Settings.console = "general"
			end
		else
			for ID, bool in pairs(conList) do
				local conInf = Inspect.Console.Detail(ID)
				if conInf.name == _qf.Settings.console then
					_qf.Settings.ID = conInf.id
					return
				end
			end
			_qf.Settings.ID = _qf.Default.ID
			_qf.Settings.console = "general"
		end
	else
		print("Could not find Console List")
	end
end

function _qf.WaitLoad(handle, units)
	local playerID = Inspect.Unit.Lookup("player")
	if playerID then
		if units[playerID] then
			-- Player is Available, set up initial system settings.
			_qf.FindConsole()
			_qf:StoreQuests()
			_qf:StoreQuestLog()
			_qf.PlayerLoaded = playerID
			_qf.PlayerAvailable = true
			-- Remove temporay handler (this one)
			Command.Event.Detach(Event.Unit.Availability.Full, _qf.WaitLoad, "Initial Player Load")
			-- Add all run-time events and handlers.
			Command.Event.Attach(Event.Unit.Availability.Full, _qf.CheckPlayer, "Player Available")
			Command.Event.Attach(Event.Unit.Availability.None, _qf.PlayerUnavail, "Check Player Unavail")
			Command.Event.Attach(Event.Unit.Availability.Partial, _qf.PlayerUnavail, "Check Player Partial")
			Command.Event.Attach(Event.System.Update.Begin, _qf.Tick, "Run Time Tick")

			if not _qf.Settings.silent then
				_qf.print('<font color="#FAFA09">[QuestFinder] <font color="#FFFFFF">'..QFind.Locale.Help)
			end
		end
	end
end

function _qf.PlayerUnavail(handle, units)
	if units[_qf.PlayerLoaded] then
		_qf.PlayerAvailable = false
	end
end

function _qf.DumpTable(Table)
	if not Table then
		return
	else
		for Key, Data in pairs(Table) do
			if type(Data) == "table" then
				print(tostring(Key)..":-")
				_qf.DumpTable(Data)
			else
				print(tostring(Key)..": "..tostring(Data))
			end
		end
	end
end

function _qf.QuestChange(handle, data)
end

function _qf.QuestAccept(handle, data)
end

function _qf.QuestAbandon(handle, data)
end

function _qf.QuestComplete(handle, data)
end

function _qf.Help()
	for id, output in pairs(QFind.Locale.HelpList) do
		_qf.print(output)
	end
end

function _qf.SlashHandler(handle, Cmd)
	if string.find(Cmd, "=") then
		local _, _, exCmd, value = string.find(Cmd, "(%a*)=(.*)")
		if exCmd then
			exCmd = string.lower(exCmd)
			if exCmd == "silent" then
				value = string.lower(value)
				if value == "true" then
					_qf.print(_qf.chatPrefix.."Start up text is now disabled.")
					_qf.Settings.silent = true
				elseif value == "false" then
					_qf.print(_qf.chatPrefix.."Start up text is now enabled.")
					_qf.Settings.silent = false
				else
					_qf.print(_qf.chatPrefix.."Unknown value for Silent: "..tostring(value))
					_qf.print(_qf.chatPrefix.."Expecting true or false")
				end
			elseif exCmd == "chat" then
				if value then
					local conList = Inspect.Console.List()
					for ID, bool in pairs(conList) do
						local conInf = Inspect.Console.Detail(ID)
						if conInf.name == value then
							_qf.print(_qf.chatPrefix.."Chat Tab now set to: "..conInf.name)
							_qf.Settings.ID = ID
							_qf.Settings.console = conInf.name
							_qf.print(_qf.chatPrefix.."Now displaying output to this chat tab.")
							return
						end
					end
					_qf.print("Unable to find chat tab (Case Sensitive): "..tostring(value))
				else
					_qf.print("Incorrect Chat Channel layout: "..tostring(value))
				end
			end
		else
			_qf.print("Unknown command: "..tostring(exCmd), tostring(value))
		end
	else
		_qf.print("Unknown command string, missing '=' : "..Cmd)
	end
end

Command.Event.Attach(Command.Slash.Register("qfind"), _qf.Search, "Quest Finder Find Slash Command")
Command.Event.Attach(Command.Slash.Register("qfHelp"), _qf.Help, "Quest Finder Help Command")
Command.Event.Attach(Command.Slash.Register("qfSetting"), _qf.SlashHandler, "Quest Finder Settings, Slash Commands")
Command.Event.Attach(Event.Unit.Availability.Full, _qf.WaitLoad, "Initial Player Load")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, _qf.VarLoad, "Assign Vars")
Command.Event.Attach(Event.Addon.SavedVariables.Save.Begin, _qf.VarSave, "Save Vars")
Command.Event.Attach(Event.Quest.Change, _qf.QuestChange, "Quest Change")
Command.Event.Attach(Event.Quest.Accept, _qf.QuestAccept, "Quest Accept")
Command.Event.Attach(Event.Quest.Abandon, _qf.QuestAbandon, "Quest Abandon")
Command.Event.Attach(Event.Quest.Complete, _qf.QuestComplete, "Quest Complete")

