RGM = {Build = {}, Data = {}, Extra = {}, Ignore = {}, Sessions = {}, Share = {}}

function RGM.Build.Engine()
	local Match, String, Upper = string.match, string.format, string.upper
	local Insert, Remove = table.insert, table.remove
	local Player = Inspect.Unit.Lookup("player")
	local Next, Pairs = next, pairs
	local Number = tonumber
	local E = RGM

	function E.Ability(Input, Type, Unit)
		local Info = {Absorbed = {}, Avoided = {}, Blocked = {}, Damage = {}, Heal = {}, Intercepted = {}, Missed = {}, Overheal = {}, Overkill = {}}
		local Pet = (Input.caster == Unit.Pet or Input.target == Unit.Pet and not Input.converted)
		local Ability = Unit.Abilities[Input.ability]
		local Name = Input.abilityName
		
		if not Ability and not Pet and E.Units({[Input.caster] = "Pet"}) then
			Pet = true
		end
		
		if Input.type then
			Input.type = Input.type:gsub("^%l", Upper)
		end
		
		if not Ability then
			if not Input.converted then
				if not Input.heal and not Input.overheal then
					Name = String("%0s (%0s)", Name, (Input.type or "Physical"))
				else
					Name = String("%0s (Heal)", Name)
				end
			end
			
			if Input.converted then
				Name = String("%0s (Absorb)", Name)
			end
			
			if Pet then
				Name = String("%0s (Pet)", Name)
			end
			
			Unit.Abilities[Input.ability] = {[Type] = Info, Tally = {}, Tooltip = Name, Update = Input.damageAvoided}
			Ability = Unit.Abilities[Input.ability]
		end
		
		if Ability then
			Ability.Tally[Type] = (Ability.Tally[Type] or 0) + 1
		end
		
		if Ability.Update then
			if Input.damage or Input.damageAbsorbed then
				Ability.Tooltip = String("%0s (%0s)%0s", Name, (Input.type or "Physical"), (Pet and " (Pet)" or ""))
				Ability.Update = nil
			end
		end
		
		if Ability and not Ability[Type] then
			Ability[Type] = Info
		end
		
		return Ability[Type]
	end

	function E.Absorb(Input)
		if not Input then return end
		
		local Ran = false
		
		for k,v in Pairs(Input) do
			local Unit = (E.Extra[k] or {Absorb = {}})
			
			for i = 1, #Unit.Absorb do
				local Absorb = Unit.Absorb[i]
				
				if Unit.Value and v and v > Unit.Value and not Absorb.Value and not Ran then
					Absorb.Value = (v - Unit.Value)
					Ran = true
				end
			end
			
			if Unit.Absorb[1] and not Unit.Absorb[1].Value then
				Unit.Absorb[1].Value = v
			end
			
			Unit.Value = (v or 0)
		end
	end

	function E.Add(Unit, Input)
		if not Input or not Unit then return end
		
		local Lookup = Inspect.Unit.Lookup(Unit)
		local Detail = Inspect.Buff.Detail
		local Ran = false
		
		if not E.Extra[Unit] then
			local Blocked = (Match(Lookup, "focus") or Match(Lookup, "mouseover") or Match(Lookup, "target"))
			
			if not Blocked then
				if Match(Lookup, "group") or Match(Lookup, "player") then
					E.Extra[Unit] = {Absorb = {}, Buff = {}, Debuff = {}, Value = 0}
				end
			end
		end
		
		for k,v in Pairs(Input) do
			if E.Ignore[v] then return end
			
			local Info = (E.Extra[Unit] and Detail(Unit, k))
			local Unit = E.Extra[Unit]
			
			if Info and E.Filter.Debuff[Info.name] and v then
				Unit.Debuff[#Unit.Debuff + 1] = {Ability = v, Icon = Info.icon, Identification = k, Name = Info.name, Remaining = Info.remaining}
				Ran = true
			end
			
			if Info and E.Filter.Buff[Info.name] and v then
				Unit.Buff[#Unit.Buff + 1] = {Ability = v, Icon = Info.icon, Identification = k, Name = Info.name, Remaining = Info.remaining}
				Ran = true
			end
			
			if Info and E.Filter.Absorb[Info.name] then
				Unit.Absorb[#Unit.Absorb + 1] = {Ability = v, Caster = Info.caster, Identification = k, Name = Info.name}
				Ran = true
			end
			
			if Info and not Ran then
				E.Ignore[v] = 0
			end
		end
	end

	function E.Blocked(Input)
		if not Input then return end
		
		for k,v in Pairs(Input) do
			if E.Data[k] and E.Instance then
				E.Data[k].Blocked = v
			end
		end
	end

	function E.Clean(Input)
		if not Input then return end
		
		local Info = {}
		
		if Input.Log and not Next(Input.Log) then
			Input.Log = nil
		end
		
		if Info then
			E.Active, E.Forced, E.Instance = nil, nil, nil
			E.Data, E.Share = {}, {}
		end
		
		for k,v in Pairs(Input) do
			local Ignore = (k == "Log" or k == "Time")
			local Abilities = (v.Abilities or Info)
			
			if v.Duration and v.Duration < 2 then
				return
			end
			
			for k,v in Pairs(Abilities) do
				local Ability = (v or Abilities[k])
				
				for k,v in Pairs(Ability) do
					local Params = (k == "Given" or k == "Taken")
					local Data = (v or Ability[k])
					
					if Params then
						for k,v in Pairs(Data) do
							if Data[k] and not Next(v) then
								Data[k] = nil
							end
						end
						
						if not Next(Data) then
							Ability[k] = nil
						end
					end
					
					if k and k == "Update" then
						Ability[k] = nil
					end
				end
				
				if not Ability.Given and not Ability.Taken then
					Abilities[k] = nil
				end
			end
			
			if not Ignore then
				if not v.Name or not v.Abilities then
					Input[k] = nil
				end
				
				if not Next(v.Log) then
					v.Log = nil
				end
			end
			
			v.Blocked = nil
			v.Combat = nil
		end
		
		return Input
	end

	function E.Combat(Input)
		if not Input then return end
		
		local Units = Inspect.Unit.List()
		local Combat = false
		
		for k,v in Pairs(Input) do
			if not Next(E.Data) and not E.Active and Units and v then
				E.Group(Units)
			end
			
			if E.Data[k] and not E.Data[k].Combat and v then
				E.Data[k].Combat = true
			end
			
			if E.Data[k] and E.Data[k].Combat and not v then
				E.Data[k].Combat = nil
			end
		end
		
		for k,v in Pairs(E.Data) do
			local Player = (Units[k] == "player" or E.Instance)
			
			if Input[v.Pet] and not v.Combat then
				v.Combat = true
			end
			
			if not Units[k] and v.Combat then
				v.Combat = nil
			end
			
			if v.Combat and Player then
				Combat = true
			end
		end
		
		for k,v in Pairs(E.Extra) do
			if not Combat and not Units[k] then
				E.Extra[k] = nil
			end
		end
		
		if not Combat and E.Active then
			E.Sessions[#E.Sessions + 1] = E.Clean(E.Data)
			E.Interface.Session(true)
		end
		
		if Combat and not E.Active then
			local Send = Command.Message.Broadcast
			local Data = String("#<%0s>", Player)
			
			Send("yell", nil, "RGM", Data)
		end
		
		E.Active = Combat
	end

	function E.Converter(Input)
		local Unit = (E.Extra[Input.target] or {Absorb = {}})
		local Heal = -Input.damageAbsorbed
		local Params = {converted = true}
		local Ran = false
		
		for k,v in Pairs(Unit.Absorb) do
			local Caster = (not Ran and E.Data[v.Caster])
			
			if Caster then
				Params.targetName = Input.targetName
				Params.casterName = Caster.Name
				Params.target = Input.target
				Params.abilityName = v.Name
				Params.ability = v.Ability
				Params.caster = v.Caster
				
				if v.Value and v.Value < Heal then
					Params.heal = v.Value
					Heal = Heal - v.Value
				else
					Params.heal = Heal
					Ran = Params.heal
				end
				
				E.Gather(Params)
			end
			
			v.Value = (v.Value and v.Value > 0 and v.Value - Heal or 0)
			Params = {converted = true}
		end
	end

	function E.Count(Ability, Event, Input, Unit)
		local Name = Input[String("%0s%0s", Unit, "Name")]
		
		if not Ability.Units then
			Ability.Units = {}
		end
		
		if not Input.crit then
			Ability["Normal Count"] = (Ability["Normal Count"] or 0) + 1
			
			if Input[Event] then
				if (not Ability["Normal Max"] or Ability["Normal Max"] < Input[Event]) then
					Ability["Normal Max"] = Input[Event]
				end
				
				if (not Ability["Normal Min"] or Ability["Normal Min"] > Input[Event]) then
					Ability["Normal Min"] = Input[Event]
				end
				
				Ability["Normal Total"] = (Ability["Normal Total"] or 0) + Input[Event]
			end
		end
		
		if Input.crit then
			Ability["Critical Count"] = (Ability["Critical Count"] or 0) + 1
			
			if Input[Event] then
				if (not Ability["Critical Max"] or Ability["Critical Max"] < Input[Event]) then
					Ability["Critical Max"] = Input[Event]
				end
				
				if (not Ability["Critical Min"] or Ability["Critical Min"] > Input[Event]) then
					Ability["Critical Min"] = Input[Event]
				end
				
				Ability["Critical Total"] = (Ability["Critical Total"] or 0) + Input[Event]
			end
		end
		
		if Event then
			Ability["Total Count"] = (Ability["Total Count"] or 0) + 1
		end
		
		if Name then
			Ability.Units[Name] = (Ability.Units[Name] or 0) + 1
		end
	end

	function E.Damage(Input)
		E.Processor(Input, O)
	end

	function E.Dodge(Input)
		E.Processor(Input, 0)
	end

	function E.Enter(Input)
		E.Combat({[Player] = "yes"})
	end

	function E.Exit(Input)
		E.Combat({[Player] = false})
	end

	function E.Force(Input, Unit, Event)
		local Force = {[Input.caster] = true, [Input.target] = true}
		local Unit = Inspect.Unit.Lookup(Unit)
		local Dummy = false
		
		if Event and Unit then
			Dummy = (Match((Input.casterName or ""), "Dummy") or Match((Input.targetName or ""), "Dummy"))
			
			if Unit == "player" or Unit == "player.pet" then
				E.Combat(Force)
			end
			
			if not Dummy and Match(Unit, "group") then
				E.Combat(Force)
			end
		end
		
		E.Forced = E.Active
	end

	function E.Gather(Input, Sync)
		local Given, Taken = false, false
		local Caster = {Abilities = {}}
		local Target = {Abilities = {}}
		
		if Input.caster and E.Data[Input.caster] then
			Caster = E.Data[Input.caster]
		end
		
		if Input.target and E.Data[Input.target] then
			Target = E.Data[Input.target]
		end
		
		if not Caster.Name then
			Caster = (E.Search(Input, "caster") or Caster)
		end
		
		if not Target.Name then
			Target = (E.Search(Input, "target") or Target)
		end
		
		if not Caster.Name and Input.caster then
			Caster = (E.Units({[Input.caster] = "Return"}) or Caster)
		end
		
		if not Target.Name and Input.target then
			Target = (E.Units({[Input.target] = "Return"}) or Target)
		end
		
		if not Input.converted and not Sync then
			if E.Instance and E.Swap(Input, Input.caster, Caster.Blocked) then
				return
			end
			
			if E.Instance and E.Swap(Input, Input.target, Target.Blocked) then
				return
			end
		end
		
		if Caster.Abilities then
			Input.casterName = (Caster.Name or Input.casterName)
			Given = E.Ability(Input, "Given", Caster)
		end
		
		if Target.Abilities then
			Input.targetName = (Target.Name or Input.targetName)
			Taken = E.Ability(Input, "Taken", Target)
		end
		
		if Input.damage then
			E.Count(Given.Damage, "damage", Input, "target")
			E.Count(Taken.Damage, "damage", Input, "caster")
		end
		
		if Input.damageAbsorbed then
			E.Count(Given.Absorbed, "damageAbsorbed", Input, "target")
			E.Count(Taken.Absorbed, "damageAbsorbed", Input, "caster")
		end
		
		if Input.damageAvoided then
			E.Count(Given.Missed, "damageAvoided", Input, "target")
			E.Count(Taken.Avoided, "damageAvoided", Input, "caster")
		end
		
		if Input.damageBlocked then
			E.Count(Given.Blocked, "damageBlocked", Input, "target")
			E.Count(Taken.Blocked, "damageBlocked", Input, "caster")
		end
		
		if Input.damageIntercepted then
			E.Count(Given.Intercepted, "damageIntercepted", Input, "target")
			E.Count(Taken.Intercepted, "damageIntercepted", Input, "caster")
		end
		
		if Input.heal then
			E.Count(Given.Heal, "heal", Input, "target")
			E.Count(Taken.Heal, "heal", Input, "caster")
		end
		
		if Input.overheal then
			E.Count(Given.Overheal, "overheal", Input, "target")
			E.Count(Taken.Overheal, "overheal", Input, "caster")
		end
		
		if Input.overkill then
			E.Count(Given.Overkill, "overkill", Input, "target")
			E.Count(Taken.Overkill, "overkill", Input, "caster")
		end
		
		if Target.Role then
			if (Input.damage or Input.damageAbsorbed or Input.heal) then
				E.Log(Input, Target)
			end
			
			if Input.damageAbsorbed then
				E.Converter(Input)
			end
		end
	end

	function E.Group(Input)
		local Instance, Raid = false, false
		local Zone = Inspect.Zone.Detail
		local Unit = Inspect.Unit.Detail
		
		for k,v in Pairs(Input) do
			local Remove = (Match(v, "focus") or Match(v, "mouseover") or Match(v, "target"))
			local Inspect = Unit(k)
			
			if not Raid and Inspect and Inspect.tier and Inspect.tier == "raid" then
				if not Match(Inspect.name, "Dummy") then
					Raid = Inspect.tier
				end
			end
			
			if not Instance and Inspect and Inspect.zone then
				Instance = Zone(Inspect.zone)
			end
			
			if v == "player" or v == "player.pet" then
				E.Units({[k] = true})
			end
			
			if Inspect and Remove then
				Input[k] = nil
			end
		end
		
		if Instance and Instance.type == "instance" or Raid then
			E.Instance = true
			E.Units(Input)
		end
	end

	function E.Heal(Input)
		E.Processor(Input, O)
	end

	function E.Log(Input, Unit)
		local Damage, Heal = false, false
		
		if not E.Data.Log then
			E.Data.Log = {}
		end
		
		if Unit and E.Data.Log then
			Damage = String("%0s's %0s Hits %0s For %0s %0s Damage", Input.casterName, Input.abilityName, Input.targetName, (Input.damage or 0), (Input.type or "Physical"))
			Heal = String("%0s's %0s Heals %0s For %0s", Input.casterName, Input.abilityName, Input.targetName, (Input.heal or 0))
		end
		
		if Input.damage or Input.damageAbsorbed then
			local Intercepted = (Input.damageIntercepted and String(" (%0s Intercepted)", Input.damageIntercepted) or "")
			local Absorbed = (Input.damageAbsorbed and String(" (%0s Absorbed)", Input.damageAbsorbed) or "")
			local Blocked = (Input.damageBlocked and String(" (%0s Blocked)", Input.damageBlocked) or "")
			local Overkill = (Input.overkill and String(" (%0s Overkill)", Input.overkill) or "")
			
			Damage = String("%0s%0s%0s%0s%0s.", Damage, Absorbed, Blocked, Intercepted, Overkill)
			
			Insert(Unit.Log, String("<font color=%0s#D20000%0s>[Damage] %0s</font>", '"', '"', Damage))
		end
		
		if Input.heal then
			Heal = String("%0s%0s.", Heal, (Input.overheal and String(" (%0s Overheal)", Input.overheal) or ""))
			
			Insert(Unit.Log, String("<font color=%0s#04B404%0s>[Heal] %0s</font>", '"', '"', Heal))
		end
		
		if Input.damage and Input.overkill then
			Insert(E.Data.Log, String("<font color=%0s#F28000%0s>%0s</font>", '"', '"', Damage))
			
			if Input.target == Player then
				print(Damage)
			end
		end
		
		if Unit and #Unit.Log >= 28 then
			Remove(Unit.Log, 1)
		end
	end

	function E.Miss(Input)
		E.Processor(Input, 0)
	end

	function E.Parry(Input)
		E.Processor(Input, 0)
	end

	function E.Processor(Input, Avoid)
		local Event = (Avoid or Input.damage or Input.damageAbsorbed)
		local Self = (Input.caster == Input.target)
		
		if not E.Active and not Self and Input.caster and Input.target then
			E.Force(Input, Input.caster, Event)
			E.Force(Input, Input.target, Event)
		end
		
		if not E.Active or not E.Gather or not Input then
			return
		end
		
		if not Avoid and Event then
			Input.damageIntercepted = (Input.damageIntercepted and -Input.damageIntercepted)
			Input.damageAbsorbed = (Input.damageAbsorbed and -Input.damageAbsorbed)
		end
		
		if Input.abilityName == "attack" or not Input.abilityName then
			Input.abilityName = "Attack"
		end
		
		if Event or Input.abilityName or Self then
			Input.ability = (Input.abilityNew or Input.abilityName)
			Input.damageAvoided = Avoid
			Input.abilityNew = nil
		end
		
		if not Avoid and Event and Self then
			Input.casterName = nil
			Input.caster = nil
		end
		
		E.Gather(Input)
	end

	function E.Receive(From, Type, Channel, Addon, Data)
		if not (Addon == "RGM") then return end
		
		local Events = {damage = 0, damageAbsorbed = 0, damageAvoided = 0, damageBlocked = 0, damageIntercepted = 0, damageModified = 0, heal = 0, overheal = 0, overkill = 0}
		local Unit = Match(Data, "#<(.-)>")
		local Info = {}
		
		if Data and Info then
			Info.abilityName = Match(Data, "abilityName<(.-)>")
			Info.casterName = Match(Data, "casterName<(.-)>")
			Info.targetName = Match(Data, "targetName<(.-)>")
			Info.ability = Match(Data, "ability<(.-)>")
			Info.caster = Match(Data, "caster<(.-)>")
			Info.target = Match(Data, "target<(.-)>")
			Info.type = Match(Data, "type<(.-)>")
			Info.crit = Match(Data, "crit<")
		end
		
		for k,v in Pairs(Events) do
			Info[k] = Number(Match(Data, String("%0s<(.-)>", k)))
		end
		
		if Unit and not E.Share[Unit] then
			E.Share[Unit] = 0
		end
		
		if Info.abilityName then
			E.Gather(Info, 0)
		end
		
		if Type == "say" then
			dump(Data)
		end
	end

	function E.Remove(Unit, Input)
		if not Input or not Unit then return end
		
		local Data = E.Extra[Unit]
		
		if Data then
			for k,v in Pairs(Data.Absorb) do
				if Input[v.Identification] then
					Remove(Data.Absorb, k)
				end
			end
			
			for k,v in Pairs(Data.Debuff) do
				if Input[v.Identification] then
					Remove(Data.Debuff, k)
				end
			end
			
			for k,v in Pairs(Data.Buff) do
				if Input[v.Identification] then
					Remove(Data.Buff, k)
				end
			end
		end
	end

	function E.Resist(Input)
		E.Processor(Input, 0)
	end

	function E.Search(Input, Unit)
		local Name = Input[String("%0s%0s", Unit, "Name")]
		
		for k,v in Pairs(E.Data) do
			if v.Pet and v.Pet == Input[Unit] then
				return E.Data[k]
			end
			
			if v.Name and v.Name == Name then
				return E.Data[k]
			end
		end
	end

	function E.Send(Input, Unit)
		if Player == Unit then return end
		
		local Send = Command.Message.Send
		local Data = ""
		
		for k,v in Pairs(Input) do
			Data = String("%0s%0s<%0s>", Data, k, v)
		end
		
		Send(Unit, "RGM", Data, tostring)
	end

	function E.Swap(Input, Unit, Blocked)
		local Event = (Input.damage or Input.damageAbsorbed or Input.damageAvoided)
		local Owner = Inspect.Unit.Lookup(Unit .. ".owner")
		
		for k,v in Pairs(E.Data) do
			if E.Share[k] and Event and Input.target == Unit and Player == Unit then
				E.Send(Input, k)
			end
			
			if E.Share[k] and Input.caster == Unit and Player == Unit then
				E.Send(Input, k)
			end
		end
		
		if Blocked and (E.Share[Unit] or E.Share[Owner]) then
			return Unit
		end
	end

	function E.Units(Input)
		local Lookup = Inspect.Unit.Lookup
		local Detail = Inspect.Unit.Detail
		local Owner = false
		local Unit = false
		
		for k,v in Pairs(Input) do
			local i = (Lookup(k .. ".owner") or k)
			
			if Detail and not E.Data[i] then
				Unit = Detail(i)
			end
			
			if Unit and Unit.nameSecondary then
				local Name = (Unit.alliance and Unit.nameSecondary or k)
				
				for k,v in Pairs(E.Data) do
					if v.Name and v.Player and Match(Name, v.Name) then
						v.Pet = Unit.id
						Owner = k
					end
				end
				
				if Owner and E.Data[Owner] then
					return E.Data[Owner]
				end
			end
			
			if v and v == "Pet" then
				return Lookup(k .. ".owner")
			end
			
			if not E.Data[i] and Unit then
				E.Data[i] = {Abilities = {}, Log = {}}
				
				E.Data[i].Calling = Unit.calling
				E.Data[i].Combat = Unit.combat
				E.Data[i].Level = Unit.level
				E.Data[i].Name = Unit.name
				E.Data[i].Player = Unit.player
				E.Data[i].Relation = Unit.relation
				
				E.Data[i].Pet = Lookup(i .. ".pet")
				E.Data[i].Blocked = Unit.blocked
				E.Data[i].Role = Unit.role
			end
			
			if v and v == "Return" then
				return E.Data[i]
			end
			
			Owner = false
			Unit = false
		end
	end

	function E.Update(Input)
		if not E.Active then return end
		
		local Time = Inspect.Time.Real()
		local Params = {}
		
		if E.Data.Time then
			E.Data.Time.Duration = Time - E.Data.Time.Start
		else
			E.Data.Time = {Duration = 0, Start = Time}
		end
		
		if E.Forced and E.Data.Time.Duration >= 0.8 then
			local Detail = Inspect.Unit.Detail
			
			for k,v in Pairs(E.Data) do
				local Info = (v.Combat and Detail(k))
				
				v.Combat = (Info and Info.combat)
			end
			
			E.Combat(Params)
			E.Forced = nil
		end
		
		E.Interface.Current()
	end

	if Next(E) then
		table.insert(Event.Buff.Add, {E.Add, "RGM", "Add Buff"})
		table.insert(Event.Buff.Remove, {E.Remove, "RGM", "Remove Buff"})
		table.insert(Event.Combat.Damage, {E.Damage, "RGM", "Damage"})
		table.insert(Event.Combat.Dodge, {E.Dodge, "RGM", "Dodge"})
		table.insert(Event.Combat.Heal, {E.Heal, "RGM", "Heal"})
		table.insert(Event.Combat.Miss, {E.Miss, "RGM", "Miss"})
		table.insert(Event.Combat.Parry, {E.Parry, "RGM", "Parry"})
		table.insert(Event.Combat.Resist, {E.Resist, "RGM", "Resist"})
		table.insert(Event.Message.Receive, {E.Receive, "RGM", "Sync"})
		table.insert(Event.System.Secure.Enter, {E.Enter, "RGM", "Enter"})
		table.insert(Event.System.Secure.Leave, {E.Exit, "RGM", "Exit"})
		table.insert(Event.System.Update.Begin, {E.Update, "RGM", "Update"})
		table.insert(Event.Unit.Detail.Absorb, {E.Absorb, "RGM", "Absorb"})
		table.insert(Event.Unit.Detail.Blocked, {E.Blocked, "RGM", "Blocked"})
		table.insert(Event.Unit.Detail.Combat, {E.Combat, "RGM", "Combat"})
	end
end