RGM.Interface = {Build = {}, Data = {}, Mouse = {}, Temp = {}}

function RGM.Build.Interface()
	local Match, String = string.match, string.format
	local Integer, Sort = math.floor, table.sort
	local Next, Pairs = next, pairs
	local I = RGM.Interface
	local G = RGM.Global
	local E = RGM

	if RGM_Global then
		G = (RGM.Global and RGM_Global)
		RGM.Global = (G and nil)
	else
		RGM_Global = RGM.Global
	end

	I.Parser = UI.CreateContext("Parser")

	I.ParserBG = UI.CreateFrame("Texture", "Parser BG", I.Parser)
	I.Units = UI.CreateFrame("Text", "Parser Units", I.Parser)
	I.Time = UI.CreateFrame("Text", "Parser Time", I.Parser)
	I.Events = UI.CreateFrame("Text", "Parser Events", I.Parser)
	I.Mini = UI.CreateFrame("Text", "Parser Mini Menu", I.Parser)
	I.Tip = UI.CreateFrame("Frame", "Parser Tooltip", I.Parser)
	I.TipBG = UI.CreateFrame("Texture", "Parser Tip BG", I.Parser)
	I.TipTX = UI.CreateFrame("Text", "Parser Tip Text", I.Parser)

	I.Parser:SetPoint("CENTER", UIParent, "CENTER", G.ParserX, G.ParserY)
	I.Parser:SetBackgroundColor(0, 0, 0, 0.8)

	I.ParserBG:SetPoint("CENTER", I.Parser, "CENTER", 0, 0)
	I.ParserBG:SetTexture("RGM", "TBGM.png")
	I.ParserBG:SetAlpha(0.08)

	I.Units:SetPoint("TOPLEFT", I.Parser, "TOPLEFT", 4, 20)
	I.Units:SetFontSize(14)
	I.Units:SetWidth(140)
	I.Units:SetLayer(2)

	I.Time:SetPoint("TOPLEFT", I.Parser, "TOPLEFT", 4, 2)
	I.Time:SetText("00:00")
	I.Time:SetFontSize(14)
	I.Time:SetLayer(2)

	I.Events:SetPoint("TOPRIGHT", I.Parser, "TOPRIGHT", 0, 28)
	I.Events:SetBackgroundColor(0, 0, 0, 0.8)
	I.Events:SetVisible(false)
	I.Events:SetFontSize(14)
	I.Events:SetLayer(4)

	I.Mini:SetPoint("TOPLEFT", I.Parser, "TOPLEFT", 0, 28)
	I.Mini:SetBackgroundColor(0, 0, 0, 0.8)
	I.Mini:SetVisible(false)
	I.Mini:SetFontSize(14)
	I.Mini:SetLayer(4)

	I.Tip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -2, -60)
	I.Tip:SetBackgroundColor(0, 0, 0, 0.8)
	I.Tip:SetVisible(false)

	I.TipBG:SetPoint("CENTER", I.Tip, "CENTER", 0, 0)
	I.TipBG:SetTexture("RGM", "TBGT.png")
	I.TipBG:SetParent(I.Tip)
	I.TipBG:SetAlpha(0.20)

	I.TipTX:SetPoint("CENTER", I.Tip, "CENTER", 0, 0)
	I.TipTX:SetParent(I.Tip)
	I.TipTX:SetFontSize(14)
	I.TipTX:SetLayer(2)

	function I.Parser.Event.LeftDown()
		local Mouse = (I.Mouse and Inspect.Mouse())
		
		if Mouse then
			I.Mouse.GParserX = G.ParserX
			I.Mouse.GParserY = G.ParserY
			I.Mouse.ParserX = Mouse.x
			I.Mouse.ParserY = Mouse.y
			I.Mouse.Parser = true
		end
	end

	function I.Parser.Event.LeftUp()
		I.Mouse = {}
	end

	function I.Parser.Event.MouseMove()
		local Mouse = (I.Mouse.Parser and Inspect.Mouse())
		
		if Mouse then
			I.Parser:SetPoint("CENTER", UIParent, "CENTER", Mouse.x - I.Mouse.ParserX + I.Mouse.GParserX, Mouse.y - I.Mouse.ParserY + I.Mouse.GParserY)
			G.ParserX = Mouse.x - I.Mouse.ParserX + I.Mouse.GParserX
			G.ParserY = Mouse.y - I.Mouse.ParserY + I.Mouse.GParserY
		end
	end

	function I.Parser.Event.WheelBack()
		if I.Data and #I.Data > G.Slots.Stop then
			G.Slots.Start = G.Slots.Start + 1
			G.Slots.Stop = G.Slots.Stop + 1
			I.Session(G.Slots.Start)
		end
	end

	function I.Parser.Event.WheelForward()
		if I.Data and G.Slots.Start > 1 then
			G.Slots.Start = G.Slots.Start - 1
			G.Slots.Stop = G.Slots.Stop - 1
			I.Session(G.Slots.Start)
		end
	end

	function I.Time.Event.LeftClick()
		I.Mini:SetVisible(not I.Mini:GetVisible())
		I.Temp.Mini = (not I.Temp.Mini or nil)
		I.Mini.Update()
	end

	function I.Events.Event.LeftClick()
		local Mode = "Mode" .. I.Temp.Mode
		local Item = I.Temp.Events
		
		if Item < 2 then
			G.Mode[I.Temp.Mode] = "DMG"
			I[Mode]:SetText("DMG")
		elseif Item < 3 then
			G.Mode[I.Temp.Mode] = "DPS"
			I[Mode]:SetText("DPS")
		elseif Item < 4 then
			G.Mode[I.Temp.Mode] = "HLG"
			I[Mode]:SetText("HLG")
		else
			G.Mode[I.Temp.Mode] = "HPS"
			I[Mode]:SetText("HPS")
		end
		
		I.Events:SetVisible(false)
		I.Temp.Events = nil
		I.Temp.Mode = nil
		I.Session(true)
	end

	function I.Events.Event.MouseMove()
		I.Select("Events", I.Events:GetTop(), I.Events:GetHeight())
	end

	function I.Mini.Change(Input)
		local Session = (I.Temp.Index or (Next(E.Data) and #E.Sessions + 1) or #E.Sessions)
		local Item = (I.Temp.Mini > 4 and I.Temp.Mini or 8)
		
		if Item < 6 then
			if Input == "Up" then
				G.Modes = (G.Modes < 14 and G.Modes or 13) + 1
			else
				G.Modes = (G.Modes > 2 and G.Modes or 2) - 1
			end
			
			I.Build.Modes()
		elseif Item < 7 then
			if Input == "Up" then
				G.Slots.Value = (G.Slots.Value == 28 and 1 or G.Slots.Value == 1 and 5 or G.Slots.Value + 1)
			else
				G.Slots.Value = (G.Slots.Value == 1 and 28 or G.Slots.Value > 5 and G.Slots.Value - 1 or 1)
			end
			
			G.Slots.Stop = G.Slots.Value
			I.Session()
		else
			if Input == "Up" then
				I.Temp.Index = (I.Temp.Index or Session) + 1
			else
				I.Temp.Index = (I.Temp.Index or Session) - 1
			end
			
			if I.Temp.Index and I.Temp.Index > #E.Sessions then
				I.Temp.Index = nil
			end
			
			if I.Temp.Index and I.Temp.Index == Session then
				I.Temp.Index = nil
			end
			
			if I.Temp.Index and I.Temp.Index < 1 then
				I.Temp.Index = nil
			end
			
			if Input == "Down" or I.Temp.Index then
				I.Session()
			end
		end
		
		if Item < 6 or I.Temp.Index then
			I.Tip:SetVisible(false)
		end
		
		I.Mini.Update()
	end

	function I.Mini.Event.LeftClick()
		local Complete = {Mode = {"ASB", "AVD", "BLK", "DMG", "DPS", "DTN", "HLG", "HPS", "HTN", "INT", "MSD", "OHG", "OKL", "OPS"}, Modes = 14, Sort = {G.Sort[1], G.Sort[2]}}
		local Item = I.Temp.Mini
		local Update = false
		
		if E.Active and Item < 3 or Item < 2 then
			I.Clear()
		end
		
		if Item < 2 then
			E.Data, E.Extra, E.Ignore, E.Sessions, I.Data = {}, {}, {}, {}, {}
			I.Tip:SetVisible(false)
		elseif Item < 3 then
			if E.Active and Next(E.Data) then
				E.Active, E.Sessions[#E.Sessions + 1] = nil, E.Clean(E.Data)
				I.Tip:SetVisible(false)
			end
		elseif Item < 4 then
			if G.Side == "hostile" then
				G.Side = "friendly"
			else
				G.Side = "hostile"
			end
			
			I.Session(true)
		elseif Item < 5 then
			I.Temp.Complete = ((not I.Temp.Complete or nil) and Complete)
			
			if not I.Temp.Complete then
				I.Parser:SetPoint("CENTER", UIParent, "CENTER", G.ParserX, G.ParserY)
			else
				I.Parser:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
			end
			
			I.Build.Modes()
			I.Session()
		else
			I.Mini.Change("Up")
			Update = true
		end
		
		if Item and Item > 0 and not Update then
			I.Mini:SetVisible(false)
			I.Temp.Mini = nil
		end
	end

	function I.Mini.Event.MouseMove()
		I.Select("Mini", I.Mini:GetTop(), I.Mini:GetHeight())
	end

	function I.Mini.Event.RightClick()
		I.Mini.Change("Down")
	end

	function I.Mini.Event.WheelBack()
		I.Mini.Change("Down")
	end

	function I.Mini.Event.WheelForward()
		I.Mini.Change("Up")
	end

	function I.Mini.Update()
		local Session = (I.Temp.Index or (Next(E.Data) and #E.Sessions + 1) or #E.Sessions)
		local Slots = (G.Slots.Value < 2 and "Self" or G.Slots.Value)
		local Text = false
		
		if G.Side and G.Side == "friendly" then
			Text = String(" %0s \n %0s \n %0s \n %0s \n %0s %0s \n %0s %0s \n %0s %0s ", "Delete Data", "New Session", "Show Enemies", "All Data Modes", "Data Modes:", G.Modes, "Unit Slots:", Slots, "Session:", Session)
		else
			Text = String(" %0s \n %0s \n %0s \n %0s \n %0s %0s \n %0s %0s \n %0s %0s ", "Delete Data", "New Session", "Show Allies", "All Data Modes", "Data Modes:", G.Modes, "Unit Slots:", Slots, "Session:", Session)
		end
		
		I.Mini:SetText(Text)
	end

	function I.Build.Modes()
		local Global = (I.Temp.Complete or G)
		local Digits = tonumber
		
		for k,v in Pairs(I) do
			local Meter = Digits(Match(k, "Meter(%d+)"))
			local Mode = Digits(Match(k, "Mode(%d+)"))
			
			if Meter or Mode then
				if Meter and Meter > Global.Modes or Mode and Mode > Global.Modes then
					I[k]:SetVisible(false)
				else
					I[k]:SetVisible(true)
				end
			end
		end
		
		for i = 1, Global.Modes do
			local Meter = "Meter" .. i
			local Mode = "Mode" .. i
			local Number = 52 * i
			
			if Meter and not I[Meter] then
				I[Meter] = UI.CreateFrame("Text", "Parser Meter" .. i, I.Parser)
				I[Meter]:SetPoint("TOPRIGHT", I.Units, "TOPRIGHT", Number, 0)
				I[Meter]:SetFontSize(14)
				I[Meter]:SetWidth(48)
				
				I[Mode] = UI.CreateFrame("Text", "Parser Mode" .. i, I.Parser)
				I[Mode]:SetPoint("TOPCENTER", I[Meter], "TOPCENTER", 0, -18)
				I[Mode]:SetFontSize(14)
			end
			
			if Meter and Mode and Number then
				Meter = I[Meter]
				Mode = I[Mode]
			end
			
			function Meter.Event.LeftClick()
				I.Tooltip(I.Search(i), Global.Mode[i])
			end
			
			function Meter.Event.MouseMove()
				I.Select("Meter", Meter:GetTop(), Meter:GetHeight())
			end
			
			function Meter.Event.MouseOut()
				I.Temp.Meter = nil
			end
			
			function Mode.Event.LeftClick()
				if I.Temp and not I.Temp.Complete then
					I.Event(i)
				end
			end
			
			function Mode.Event.RightClick()
				local Mode = (Global.Sort and I.Search(i))
				
				if Mode and Mode.Event and Mode.Type then
					Global.Sort[1] = Mode.Event
					Global.Sort[2] = Mode.Type
				end
				
				I.Session(true)
			end
			
			Mode:SetText(Global.Mode[i])
		end
		
		I.Parser:SetWidth(I.Units:GetWidth() + 6 + (48 * Global.Modes) + (4 * Global.Modes))
		I.ParserBG:SetWidth(I.Parser:GetWidth() - 8)
	end

	function I.Clear()
		local G = (I.Temp.Complete or G)
		
		I.Time:SetText("00:00")
		I.Units:SetText("")
		
		for i = 1, G.Modes do
			I["Meter" .. i]:SetText("")
		end
		
		I.Parser:SetHeight(I.Units:GetHeight() + 20.00)
		I.ParserBG:SetHeight(I.Parser:GetHeight() - 8)
	end

	function I.Current()
		if I.Temp.Index then return end
		
		local Time = E.Data.Time
		
		if Time and Time.Duration - (Time.Update or -2) > 1 then
			Time.Update = Time.Duration
			I.Update(E.Data)
		end
		
		if Time and Time.Duration < 0.02 then
			G.Slots.Stop = G.Slots.Value
			G.Slots.Start = 1
			I.Mini.Update()
		end
	end

	function I.Duration(Input)
		local Hour = Integer(Input.Time.Duration / 60 / 60)
		local Min = Integer(Input.Time.Duration / 60 % 60)
		local Sec = Input.Time.Duration % 60
		
		if Hour > 0 then
			return String("%02d:%02d:%02d", Hour, Min, Sec)
		else
			return String("%02d:%02d", Min, Sec)
		end
	end

	function I.Event(Input)
		I.Events:SetText(String(" %0s \n %0s \n %0s \n %0s", "Damage", "Damage P/S", "Healing", "Healing P/S"))
		
		if not I.Events:GetVisible() then
			I.Temp.Mode = Input
		else
			I.Temp.Mode = nil
		end
		
		I.Events:SetVisible(not I.Events:GetVisible())
	end

	function I.Search(Input)
		local Data = {Event = false, Type = false}
		local G = (I.Temp.Complete or G)
		
		if G.Mode[Input] == "ASB" then
			Data.Event = "Absorbed"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "AVD" then
			Data.Event = "Avoided"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "BLK" then
			Data.Event = "Blocked"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "DMG" then
			Data.Event = "Damage"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "DPS" then
			Data.Event = "Damage"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "DTN" then
			Data.Event = "Damage"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "HLG" then
			Data.Event = "Heal"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "HPS" then
			Data.Event = "Heal"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "HTN" then
			Data.Event = "Heal"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "INT" then
			Data.Event = "Intercepted"
			Data.Type = "Taken"
		end
		
		if G.Mode[Input] == "MSD" then
			Data.Event = "Missed"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "OHG" then
			Data.Event = "Overheal"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "OKL" then
			Data.Event = "Overkill"
			Data.Type = "Given"
		end
		
		if G.Mode[Input] == "OPS" then
			Data.Event = "Overheal"
			Data.Type = "Given"
		end
		
		return Data
	end

	function I.Select(Frame, Top, Height)
		local Mouse = (Frame and Inspect.Mouse())
		
		if Frame == "Events" and Height then
			I.Temp[Frame] = (Mouse.y + 18 - Top) / (Height / 4)
		end
		
		if Frame == "Mini" and Height then
			I.Temp[Frame] = (Mouse.y + 18 - Top) / (Height / 7)
		end
		
		if Frame == "Meter" and Height then
			I.Temp[Frame] = (Mouse.y + 20 - Top) / (Height / (#I.Data < G.Slots.Value and (#I.Data + 1) or G.Slots.Value))
		end
	end

	function I.Session(Scroll, Return, Tip)
		local Session = (E.Active and E.Data or E.Sessions[#E.Sessions])
		
		if I.Temp.Index then
			Session = E.Sessions[I.Temp.Index]
		end
		
		if not Scroll then
			G.Slots.Stop = G.Slots.Value
			G.Slots.Start = 1
		end
		
		if Return then
			I.Update(Session, Tip)
			return Session
		end
		
		I.Update(Session)
	end

	function I.Short(Input)
		if not Input then return end
		
		if Input < 10000 then
			return String("%d", Input)
		elseif Input < 100000 then
			return String("%5.1fk", Input / 1000)
		elseif Input < 1000000 then
			return String("%4dk", Input / 1000)
		elseif Input < 10000000 then
			return String("%4.2fm", Input / 1000000)
		elseif Input < 100000000 then
			return String("%4.1fm", Input / 1000000)
		elseif Input < 1000000000 then
			return String("%dm", Input / 1000000)
		else
			return String("%0s", 0)
		end
	end

	function I.Sort(A, B)
		local G = (I.Temp.Complete or G)
		local Event = G.Sort[1]
		local Type = G.Sort[2]
		
		if not A or not A[Type] or not A[Type][Event] then
			A = {[Type] = {[Event] = 0}}
		end
		
		if not B or not B[Type] or not B[Type][Event] then
			B = {[Type] = {[Event] = 0}}
		end
		
		return A[Type][Event] > B[Type][Event]
	end

	function I.Tooltip(Input, Mode, Advanced)
		local Abilities, Count, Text, Units = {}, {}, {}, {}
		local Gsub, Upper = string.gsub, string.upper
		local Index = Integer(I.Temp.Meter or 0)
		local Session = I.Session(0, 0, 0)
		local Data = I.Data[Index]
		
		if I.Tip:GetVisible() then
			return I.Tip:SetVisible(false)
		end
		
		if Data and Session then
			local Total = Data[Input.Type][Input.Event]
			local Unit = Session[Data.Detail.Unit]
			local Time = Session.Time.Duration
			
			for k,v in Pairs(Unit.Abilities) do
				if v[Input.Type] and v[Input.Type][Input.Event] and v[Input.Type][Input.Event].Units then
					Abilities[#Abilities + 1] = {Ability = k, Total = (v[Input.Type][Input.Event]["Critical Total"] or 0) + (v[Input.Type][Input.Event]["Normal Total"] or 0)}
					Abilities.Total = (Abilities.Total or 0) + (v[Input.Type][Input.Event]["Total Count"] or 0)
					
					if not (Mode == "ASB" or Mode == "AVD" or Mode == "BLK" or Mode == "MSD") then
						Abilities.Critical = (Abilities.Critical or 0) + (v[Input.Type][Input.Event]["Critical Count"] or 0)
						Abilities.Normal = (Abilities.Normal or 0) + (v[Input.Type][Input.Event]["Normal Count"] or 0)
					end
					
					if Mode == "AVD" or Mode == "MSD" then
						Abilities[#Abilities].Total = (v[Input.Type][Input.Event]["Total Count"] or 0)
					end
					
					if Match(Mode, "%aPS") then
						Abilities[#Abilities].Total = Abilities[#Abilities].Total / Time
					end
					
					for k,v in Pairs(v[Input.Type][Input.Event].Units) do
						Count[k] = (Count[k] or 0) + v
					end
				end
				
				if v[Input.Type] and not Next(v[Input.Type]["Heal"] or v[Input.Type]["Overheal"] or Units) then
					Abilities.Tally = (Abilities.Tally or 0) + (v.Tally[Input.Type] or 0)
				end
			end
			
			for k,v in Pairs(Count) do
				Units[#Units + 1] = {Name = k, Total = v}
			end
			
			if #Abilities > 0 then
				Sort(Abilities, function(A, B) return A.Total > B.Total end)
			else
				return
			end
			
			if #Units > 0 then
				Sort(Units, function(A, B) return A.Total > B.Total end)
			else
				return
			end
			
			if Match(Mode, "%aPS") then
				Total = Total / Time
			end
			
			for k,v in Pairs(Abilities) do
				if not Match(k, "%a+") and v.Ability then
					if not Match(Unit.Abilities[v.Ability].Tooltip, "(Pet)") then
						Text.Unit = String("%0s  %-8s\t%-10s\t%0s  \n", (Text.Unit or ""), Gsub(I.Short(v.Total), "%s+", ""), String("[%.2f%%]", v.Total / Total * 100), Unit.Abilities[v.Ability].Tooltip)
					else
						Text.Pet = String("%0s  %-8s\t%-10s\t%0s  \n", (Text.Pet or ""), Gsub(I.Short(v.Total), "%s+", ""), String("[%.2f%%]", v.Total / Total * 100), Unit.Abilities[v.Ability].Tooltip)
					end
				end
			end
			
			for k,v in Pairs(Units) do
				Text.Count = String("%0s  %-8s\t%-10s\t%0s  \n", (Text.Count or ""), v.Total, String("[%.2f%%]", v.Total / Abilities.Total * 100), v.Name)
			end
			
			if not Advanced then
				local Detail = String("%0s's [%0s] [%0s] Basic Ability Breakdown", Data.Detail.Name, Gsub((Unit.Calling or "NPC"), "^%l", Upper), Mode)
				local Info = false
				
				if Mode == "ASB" or Mode == "AVD" or Mode == "BLK" or Mode == "MSD" then
					Info = String("[Attack Count: %0s] [%0s Count: %0s] [%.2f%%]", Abilities.Tally, Input.Event, Abilities.Total, Abilities.Total / Abilities.Tally * 100)
				else
					Info = String("[Normal Count: %0s] [Critical Count: %0s] [%.2f%%]", Abilities.Normal, Abilities.Critical, Abilities.Critical / Abilities.Total * 100)
				end
				
				Text.Out = String("  %0s  \n\n  %0s  \n\n%0s\n%0s%0s", Detail, Info, Text.Count, Text.Unit, (Text.Pet and "\n" .. Text.Pet or ""))
			else
				
			end
		end
		
		if #Abilities > 0 and Text.Out then
			I.TipTX:SetText(Text.Out)
			I.Tip:SetVisible(true)
		end
		
		if I.Tip:GetVisible() then
			I.Tip:SetHeight(I.TipTX:GetHeight() + 16.00)
			I.Tip:SetWidth(I.TipTX:GetWidth() + 16.00)
			I.TipBG:SetHeight(I.Tip:GetHeight() - 8)
			I.TipBG:SetWidth(I.Tip:GetWidth() - 8)
		end
	end

	function I.Update(Input, Tip)
		if not Input then return end
		
		local Player = Inspect.Unit.Lookup("player")
		local Time = Input.Time.Duration
		local Total = {"Total"}
		local Text = {}
		
		if Time and Time - 0.2 < 0 then
			Time = 0.88
		end
		
		if #I.Data > 0 then
			I.Data = {}
		end
		
		for k,v in Pairs(Input) do
			local Run = (v.Name and true)
			
			if v.Name and not v.Relation then
				v.Relation = "hostile"
			end
			
			if G.Slots.Value == 1 then
				if G.Side == "friendly" and not (k == Player) then
					Run = (v.Name and false)
				else
					G.Slots.Stop = 5
				end
			end
			
			if G.Side and G.Side == v.Relation and Run then
				I.Data[#I.Data + 1] = {Detail = {Name = v.Name, Unit = k}, Given = {}, Taken = {}}
				
				for k,v in Pairs(v.Abilities) do
					for k,v in Pairs(v.Given or {}) do
						if not (k == "Avoided" or k == "Missed") then
							I.Data[#I.Data].Given[k] = (I.Data[#I.Data].Given[k] or 0) + (v["Critical Total"] or 0) + (v["Normal Total"] or 0)
						else
							I.Data[#I.Data].Given[k] = (I.Data[#I.Data].Given[k] or 0) + (v["Total Count"] or 0)
						end
					end
					
					for k,v in Pairs(v.Taken or {}) do
						if not (k == "Avoided" or k == "Missed") then
							I.Data[#I.Data].Taken[k] = (I.Data[#I.Data].Taken[k] or 0) + (v["Critical Total"] or 0) + (v["Normal Total"] or 0)
						else
							I.Data[#I.Data].Taken[k] = (I.Data[#I.Data].Taken[k] or 0) + (v["Total Count"] or 0)
						end
					end
				end
			end
		end
		
		if #I.Data > 0 then
			Sort(I.Data, I.Sort)
		end
		
		for k,v in Pairs(I.Data) do
			local Valid = (k >= G.Slots.Start and k <= G.Slots.Stop)
			local G = (I.Temp.Complete or G)
			
			for i = 1, G.Modes do
				local Mode = I.Search(i)
				local Output = false
				
				if Mode and Mode.Event and Mode.Type then
					Output = (v[Mode.Type] and v[Mode.Type][Mode.Event])
				end
				
				if Output and Match(G.Mode[i], "%aPS") then
					Output = Output / Time
				end
				
				if Valid then
					Text[i + 1] = String("%0s%5s\n", (Text[i + 1] or ""), I.Short(Output or 0))
				end
				
				Total[i + 1] = (Total[i + 1] or 0) + (Output or 0)
			end
			
			if Valid then
				Text[1] = String("%0s%0s\n", (Text[1] or ""), v.Detail.Name)
			end
		end
		
		if #Text > 0 and not Tip then
			local G = (I.Temp.Complete or G)
			
			for i = 1, G.Modes do
				if Text[i + 1] and I["Meter" .. i] then
					I["Meter" .. i]:SetText(String("%0s%5s", Text[i + 1], I.Short(Total[i + 1])))
				end
			end
			
			I.Units:SetText(String("%0s%0s", Text[1], Total[1]))
			I.Parser:SetHeight(I.Units:GetHeight() + 20.40)
			I.ParserBG:SetHeight(I.Parser:GetHeight() - 8)
			I.Time:SetText(I.Duration(Input))
		end
	end

	I.Parser:SetHeight(I.Units:GetHeight() + 20.00)
	I.ParserBG:SetHeight(I.Parser:GetHeight() - 8)
	I.Build.Modes()
end