------------------------------------------------------------------
------------------ Super Meter - Data Processor ------------------
------------------------------------------------------------------

function SuperMeter.Clean(session)
	if not SuperMeter_enabled or not session then return end

	if not session.clean then
		for k1,v1 in pairs(session.units) do
			if not k1 or not v1 then return end
			
			if v1.relation == "friendly" then
				if v1.abilities and not v1.player then
					session.units[k1] = nil
				end
			end
			
			if k1 == "Unknown" or not v1.name then
				session.units[k1] = nil
			end
			
			for k2,v2 in pairs(v1.casters) do
				if v1.casters[k2] and not next(v2) then
					v1.casters[k2] = nil
				end
			end
			
			for k2,v2 in pairs(v1.targets) do
				if v1.targets[k2] and not next(v2) then
					v1.targets[k2] = nil
				end
			end
			
			for k2,v2 in pairs(v1.abilities) do
				local ability = not not v1.abilities[k2]
				local count = 0
				
				for k3,v3 in pairs(v2) do
					if not v3.value or v3.value == 0 then
						v2[k3] = nil
					end
					if ability and v2[k3] then
						count = count + 1
					end
				end
				
				if count == 0 then
					v1.abilities[k2] = nil
				end
			end
		end
		
		session.clean = true
	end

	if session.clean then
		session.enemies = {}
		session.allies = {}
	end
end

function SuperMeter.Data()
	if not SuperMeter_enabled then return end

	local session = SuperMeter.Session()
	local time = session.combatTime
	local mode1 = SuperMeter_mode1
	local mode2 = SuperMeter_mode2
	local mode = SuperMeter_mode
	local side = SuperMeter_side

	local units = {}
	local data = {}

	if session then
		session.enemies = {}
		session.allies = {}
	end

	for k,v in pairs(session.units) do
		if v and not v.relation then
			v.relation = "hostile"
		end
		
		if v and v.name then
			data.identity = v.identity
			data.calling = v.calling
			data.level = v.level
			data.name = v.name
		end
		
		if data.name then
			if mode1 == "DPS" then
				data.mode1 = v.damage / time
			elseif mode1 == "DMG" then
				data.mode1 = v.damage
			elseif mode1 == "HPS" then
				data.mode1 = v.healing / time
			elseif mode1 == "HLG" then
				data.mode1 = v.healing
			elseif mode1 == "OPS" then
				data.mode1 = v.overHealing / time
			elseif mode1 == "OHG" then
				data.mode1 = v.overHealing
			elseif mode1 == "OKL" then
				data.mode1 = v.overKill
			elseif mode1 == "DTN" then
				data.mode1 = v.damageTaken
			elseif mode1 == "HTN" then
				data.mode1 = v.healingTaken
			elseif mode1 == "ASB" then
				data.mode1 = v.damageAbsorbed
			elseif mode1 == "AVD" then
				data.mode1 = v.damageAvoided
			elseif mode1 == "BLK" then
				data.mode1 = v.damageBlocked
			elseif mode1 == "DFT" then
				data.mode1 = v.damageDeflected
			elseif mode1 == "MSD" then
				data.mode1 = v.damageMissed
			end
			
			if mode2 == "DPS" then
				data.mode2 = v.damage / time
			elseif mode2 == "DMG" then
				data.mode2 = v.damage
			elseif mode2 == "HPS" then
				data.mode2 = v.healing / time
			elseif mode2 == "HLG" then
				data.mode2 = v.healing
			elseif mode2 == "OPS" then
				data.mode2 = v.overHealing / time
			elseif mode2 == "OHG" then
				data.mode2 = v.overHealing
			elseif mode2 == "OKL" then
				data.mode2 = v.overKill
			elseif mode2 == "DTN" then
				data.mode2 = v.damageTaken
			elseif mode2 == "HTN" then
				data.mode2 = v.healingTaken
			elseif mode2 == "ASB" then
				data.mode2 = v.damageAbsorbed
			elseif mode2 == "AVD" then
				data.mode2 = v.damageAvoided
			elseif mode2 == "BLK" then
				data.mode2 = v.damageBlocked
			elseif mode2 == "DFT" then
				data.mode2 = v.damageDeflected
			elseif mode2 == "MSD" then
				data.mode2 = v.damageMissed
			end
			
			if mode == mode1 then
				data.mode = data.mode1
			else
				data.mode = data.mode2
			end
			
			if side == "Allies" then
				if v.relation == "friendly" and v.player then
					table.insert(units, data)
				end
			else
				if v.relation == "hostile" and v.name then
					table.insert(units, data)
				end
			end
			
			data = {}
		end
	end

	if mode == mode1 then
		table.sort(units, function(a,b) return a.mode1 > b.mode1 end)
	else
		table.sort(units, function(a,b) return a.mode2 > b.mode2 end)
	end

	if side == "Enemies" then
		session.enemies = units
	else
		session.allies = units
	end

	if #units > 28 then
		for i = 28 + 1, #units do
			units[i] = nil
		end
	end
end

function SuperMeter.Entry(unit)
	if not SuperMeter_logEnabled then return end
	if not SuperMeter_logShowing then return end
	if not SuperMeter_enabled then return end

	local interface = SuperMeter.Interface
	local session = SuperMeter.Session()
	local display = SuperMeter_unit
	local format = string.format
	local target = unit

	local title = ""
	local data = ""

	if display then
		for k,v in pairs(session.units) do
			if session.units[k].name == display then
				target = session.units[k]
			end
		end
	end

	if not display and not unit then
		local detail = Inspect.Unit.Detail("player.target")
		if detail and session.units[detail.id] then
			target = session.units[detail.id]
		end
	end

	if target and target.combatlog[1] then
		for k,v in pairs(target.combatlog) do
			data = format("%0s%0s\n", data, v)
		end
		
		if data and title then
			title = format("%0s 's Combat Log", target.name)
			SuperMeter_table = target.combatlog
			SuperMeter_unit = target.name
		end
	else
		for k,v in pairs(session.deathlog) do
			data = format("%0s%0s\n", data, v)
		end
		
		if data and title then
			SuperMeter_table = session.deathlog
			title = "Session Death Log"
		end
	end

	interface.LogTitle(title)
	interface.LogText(data)
end

function SuperMeter.Force(params)
	if params.casterName == params.targetName then return end
	if params.heal or params.overheal then return end

	local session = SuperMeter.Session(true)
	local filters = SuperMeter.Filters()
	local now = Inspect.Time.Real()
	local timer = SuperMeter.Timer

	local time = false

	if params.caster and params.target then
		local lookup = Inspect.Unit.Lookup(params.caster)
		
		if session and lookup and now then
			local mouseover = string.match(lookup, "mouseover")
			local target = string.match(lookup, "target")
			local focus = string.match(lookup, "focus")
			
			if not mouseover and not target and not focus then
				time = now - timer
			end
		end
		
		if time and time >= 0.88 then
			local unit = Inspect.Unit.Detail(params.target)
			if lookup == "player" or lookup == "player.pet" or unit and unit.combat and unit.tier == "raid" and not filters.tier[unit.name] and SuperMeter_group then
				SuperMeter_forced = true
				SuperMeter.Enter()
			end
		end
	end
end

function SuperMeter.Gather(params, event, caster, target)
	if not SuperMeter_enabled then return end

	local data = {damage = 0, damageAbsorbed = 0, damageAvoided = 0, damageBlocked = 0, damageDeflected = 0, damageMissed = 0, damageTaken = 0, healing = 0, healingTaken = 0, overHealing = 0, overKill = 0}

	local converter = (SuperMeter_converter and params.damageAbsorbed)
	local hitless = (event == "avoid" or event == "miss")
	local session = SuperMeter.Session(true)
	local filters = SuperMeter.Filters()
	local log = SuperMeter_logEnabled
	local player = SuperMeter.Player
	local name = params.abilityName
	local warfront = SuperMeter_pvp
	local range = SuperMeter.Range
	local boss = SuperMeter_boss
	local format = string.format
	local full = SuperMeter_full
	local send = SuperMeter.Send
	local sync = SuperMeter.Sync
	local match = string.match
	local upper = string.upper

	local advanced = false
	local type = false
	local pet = false

	local hits = 0

	if not filters or not session or session.clean or session.split then return end

	for k,v in pairs(session.units) do
		local unit = (sync[k] and v.player and v.name)
		if range[k] and unit and (params.caster == player.id or params.target == player.id) and not (event == "absorb") then
			send(params, event, unit)
		end
		if params.caster == session.units[k].pet then
			pet = true
		end
	end

	if event then
		params.damageIntercepted = (params.damageIntercepted and -params.damageIntercepted)
		params.damageAbsorbed = (params.damageAbsorbed and -params.damageAbsorbed)
		params.damageModified = (params.damageModified and -params.damageModified)
	end

	if data and full then
		if params.damage or params.damageAbsorbed or params.damageIntercepted then
			params.damage = SuperMeter.Full(params, 0)
		end
		if params.heal or params.overheal then
			params.heal = SuperMeter.Full(params, 0)
		end
	end

	if target and target.player and target.combatlog and not warfront then
		advanced = true
	end

	if not name and params.abilityNew then
		local ability = Inspect.Ability.New.Detail(params.abilityNew)
		if ability and ability.id and ability.name then
			params.abilityName = ability.name
			name = ability.name
		end
	end

	if not name or not match(name, "%a+") or name == "attack" then
		params.abilityName = "Attack"
		name = "Attack"
	end

	if event == "damage" then
		if params.type and not hitless then
			params.type = params.type:gsub("^%l", upper)
		else
			params.type = "Physical"
		end
	end

	if event and pet and params.caster then
		name = format("%0s (Pet)", params.abilityName)
	end

	if event and event == "damage" then
		type = format(" (%0s)", params.type)
	end

	if event and event == "absorb" then
		type = " (Absorb)"
	end

	if event and event == "avoid" then
		type = " (Avoid)"
	end

	if event and event == "miss" then
		type = " (Miss)"
	end

	if event and event == "heal" then
		type = " (Heal)"
	end

	if not caster.abilities[name] then
		caster.abilities[name] = {}
		
		local new = caster.abilities[name]
		
		for k,v in pairs(data) do
			new[k] = {count = 0, critCount = 0, critLess = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0}
		end
	end

	if not target.abilities[name] then
		target.abilities[name] = {}
		
		local new = target.abilities[name]
		
		for k,v in pairs(data) do
			new[k] = {count = 0, critCount = 0, critLess = 0, critMax = 0, critMin = 0, max = 0, min = 0, value = 0}
		end
	end

	if caster.abilities[name] then
		local events = {damage = params.damage, healing = params.heal, overHealing = params.overheal, overKill = params.overkill}
		local ability = caster.abilities[name]
		
		if ability and hitless then
			ability.damageMissed.count = ability.damageMissed.count + 1
			ability.damageMissed.value = ability.damageMissed.value + 1
			caster.damageMissed = caster.damageMissed + 1
			ability.damageMissed.tooltip = type
		end
		
		for k,v in pairs(events) do
			local critical = format("%0sCritical", k)
			
			if ability and not filters[k][params.abilityName] then
				if filters[critical] and filters[critical][params.abilityName] or not params.abilityNew then
					ability[k].critLess = ability[k].critLess + 1
				end
				
				if params.crit then
					ability[k].critCount = ability[k].critCount + 1
					
					if v < ability[k].critMin or ability[k].critMin <= 0 then
						ability[k].critMin = v
					end
					
					if v > ability[k].critMax then
						ability[k].critMax = v
					end
				else
					ability[k].count = ability[k].count + 1
					
					if v < ability[k].min or ability[k].min <= 0 then
						ability[k].min = v
					end
					
					if v > ability[k].max then
						ability[k].max = v
					end
				end
				
				ability[k].value = ability[k].value + v
				ability[k].tooltip = type
				caster[k] = caster[k] + v
			end
		end
	end

	if target.abilities[name] then
		local events = {damageAbsorbed = params.damageAbsorbed, damageBlocked = params.damageBlocked, damageDeflected = params.damageDeflected, damageTaken = params.damage, healingTaken = params.heal}
		local ability = target.abilities[name]
		
		if ability and hitless then
			ability.damageAvoided.count = ability.damageAvoided.count + 1
			ability.damageAvoided.value = ability.damageAvoided.value + 1
			target.damageAvoided = target.damageAvoided + 1
			ability.damageAvoided.tooltip = type
		end
		
		for k,v in pairs(events) do
			local critical = format("%0sCritical", k)
			
			if ability and not filters[k][params.abilityName] then
				if filters[critical] and filters[critical][params.abilityName] or not params.abilityNew then
					ability[k].critLess = ability[k].critLess + 1
				end
				
				if params.crit then
					ability[k].critCount = ability[k].critCount + 1
					
					if v < ability[k].critMin or ability[k].critMin <= 0 then
						ability[k].critMin = v
					end
					
					if v > ability[k].critMax then
						ability[k].critMax = v
					end
				else
					ability[k].count = ability[k].count + 1
					
					if v < ability[k].min or ability[k].min <= 0 then
						ability[k].min = v
					end
					
					if v > ability[k].max then
						ability[k].max = v
					end
				end
				
				ability[k].value = ability[k].value + v
				ability[k].tooltip = type
				target[k] = target[k] + v
			end
		end
	end

	if caster.name and target.casters and not warfront then
		local events = {damage = params.damage, damageAbsorbed = params.damageAbsorbed, damageBlocked = params.damageBlocked, damageDeflected = params.damageDeflected, healing = params.heal}
		local name = (params.casterName or caster.name or "")
		
		if caster.calling then
			name = format("%0s (%0s)", caster.name, caster.calling:gsub("^%l", upper))
		end
		
		if caster.level == "??" then
			name = format("%0s (Boss)", caster.name)
		end
		
		if hitless then
			if target.casters.hitless[name] then
				target.casters.hitless[name] = target.casters.hitless[name] + 1
			else
				target.casters.hitless[name] = 1
			end
		end
		
		for k,v in pairs(events) do
			if target.casters[k][name] then
				target.casters[k][name] = target.casters[k][name] + 1
			else
				target.casters[k][name] = 1
			end
		end
	end

	if target.name and caster.targets and not warfront then
		local events = {damage = params.damage, healing = params.heal, overHealing = params.overheal, overKill = params.overkill}
		local name = (params.targetName or target.name or "")
		
		if target.calling then
			name = format("%0s (%0s)", target.name, target.calling:gsub("^%l", upper))
		end
		
		if target.level == "??" then
			name = format("%0s (Boss)", target.name)
		end
		
		if hitless then
			if caster.targets.hitless[name] then
				caster.targets.hitless[name] = caster.targets.hitless[name] + 1
			else
				caster.targets.hitless[name] = 1
			end
		end
		
		for k,v in pairs(events) do
			if caster.targets[k][name] then
				caster.targets[k][name] = caster.targets[k][name] + 1
			else
				caster.targets[k][name] = 1
			end
		end
	end

	if not boss then
		if filters.boss[target.name] then
			for k,v in pairs(target.abilities) do
				hits = hits + v.damageTaken.count + v.damageTaken.critCount
			end
		end
		
		if not boss and hits >= 12 then
			print(format("%0s %0s", target.name, SuperMeter_region22))
			SuperMeter_split = true
			SuperMeter_boss = true
			SuperMeter.Combat()
		end
	end

	if advanced and event and log then
		SuperMeter.Log(params, caster, target)
	end

	if advanced and converter then
		SuperMeter.Absorb(params, target)
	end
end

function SuperMeter.Log(params, caster, target)
	if not SuperMeter_enabled then return end

	local targetName = string.format("[%0s]", target.name)
	local session = SuperMeter.Session(true)
	local player = SuperMeter.Player
	local format = string.format
	local unit = SuperMeter_unit
	local sub = string.gsub

	local data = {}

	if data and params then
		data.intercepted = (params.damageIntercepted and format("(%0s Intercepted)", params.damageIntercepted))
		data.deflected = (params.damageDeflected and format("(%0s Deflected)", params.damageDeflected))
		data.absorbed = (params.damageAbsorbed and format("(%0s Absorbed)", params.damageAbsorbed))
		data.modified = (params.damageModified and format("(%0s Modified)", params.damageModified))
		data.time = (session.combatTime and format("[%0s]", SuperMeter.Time(session.combatTime)))
		data.blocked = (params.damageBlocked and format("(%0s Blocked)", params.damageBlocked))
		data.overheal = (params.overheal and format("(%0s Overheal)", params.overheal))
		data.overkill = (params.overkill and format("(%0s Overkill)", params.overkill))
		data.ability = (params.abilityName and format("%0s", params.abilityName))
		data.damageType = (params.type and format("%0s Damage", params.type))
		data.damage = (params.damage and format("%0s", params.damage))
		data.caster = (caster.name and format("%0s's", caster.name))
		data.target = (target.name and format("%0s", target.name))
		data.heal = (params.heal and format("%0s", params.heal))
		data.critical = (params.crit and "Critically")
	end

	if params.damage then
		data.info1 = sub(format("%0s %0s %0s %0s Hits %0s For %0s %0s %0s %0s %0s %0s %0s %0s.", data.time, data.caster, data.ability, data.critical, data.target, data.damage, data.damageType, data.absorbed, data.blocked, data.deflected, data.intercepted, data.modified, data.overkill), " nil ", " ")
		data.info1 = format("<font color=%0s#D20000%0s>%0s</font>", '"', '"', sub(data.info1, " nil", ""))
		table.insert(target.combatlog, data.info1)
	end

	if params.damage and params.overkill then
		data.info2 = sub(format("%0s %0s %0s %0s %0s Hits %0s For %0s %0s %0s %0s %0s %0s %0s %0s.", data.time, targetName, data.caster, data.ability, data.critical, data.target, data.damage, data.damageType, data.absorbed, data.blocked, data.deflected, data.intercepted, data.modified, data.overkill), " nil ", " ")
		data.info2 = format("<font color=%0s#F28000%0s>%0s</font>", '"', '"', sub(data.info2, " nil", ""))
		table.insert(session.deathlog, data.info2)
	end

	if params.heal then
		data.info3 = sub(format("%0s %0s %0s %0s Heals %0s For %0s %0s.", data.time, data.caster, data.ability, data.critical, data.target, data.heal, data.overheal), " nil ", " ")
		data.info3 = format("<font color=%0s#04B404%0s>%0s</font>", '"', '"', sub(data.info3, " nil", ""))
		table.insert(target.combatlog, data.info3)
	end

	if params.damage and params.overkill and target.name == player.name then
		Command.Console.Display("general", true, format("[Death] %0s", data.info1), true)
	end

	if data.ability and #target.combatlog > 48 then
		table.remove(target.combatlog, 1)
	end

	if unit == target.name or not unit then
		SuperMeter.Entry()
	end
end

function SuperMeter.Number(number)
	if not SuperMeter_enabled then return end

	local format = string.format

	if number < 10000 then
		return format("%d", number)
	elseif number < 100000 then
		return format("%4.1fk", number / 1000)
	elseif number < 1000000 then
		return format("%dk", number / 1000)
	elseif number < 10000000 then
		return format("%4.2fm", number / 1000000)
	elseif number < 100000000 then
		return format("%4.1fm", number / 1000000)
	else
		return format("%dm", number / 1000000)
	end
end

function SuperMeter.Send(params, event, unit)
	if not SuperMeter_enabled or params.sync or params.sent or not unit then return end
	if params then params.ability = nil end

	local debug = SuperMeter.Debug
	local format = string.format

	local data = {}

	if not data.text then
		data.text = format("<event> <%0s>%0s", event, (data.text or ""))
	end

	for k,v in pairs(params) do
		data.text = format("%0s <%0s> <%0s>", (data.text or ""), k, v)
	end

	if data.text then
		Command.Message.Send(unit, "sync", data.text, debug)
		params.sent = true
	end
end

function SuperMeter.Session(current, new)
	if not SuperMeter_enabled then return end

	local session = {allies = {}, combatTime = 0, deathlog = {}, enemies = {}, startCombat = 0, units = {}}
	local offset = #SuperMeter.Sessions - SuperMeter.Offset
	local sessions = SuperMeter.Sessions
	local index = #SuperMeter.Sessions

	if current then
		if sessions[index] then
			return sessions[index]
		else
			return session
		end
	end

	if not new then
		if sessions[offset] then
			return sessions[offset]
		else
			return session
		end
	end

	if new then
		return session
	end
end

function SuperMeter.Time(time)
	if not SuperMeter_enabled then return end

	local hour = math.floor(time / 60 / 60)
	local min = math.floor(time / 60 % 60)
	local format = string.format
	local sec = time % 60

	if hour > 0 then
		return format("%02d:%02d:%02d", hour, min, sec)
	else
		return format("%02d:%02d", min, sec)
	end
end

function SuperMeter.Unit(params, entity)
	if not SuperMeter_enabled then return end

	if not SuperMeter_combat and not SuperMeter_pvp then
		SuperMeter.Force(params)
	end

	local session = SuperMeter.Session(true)
	local instance = SuperMeter_instance
	local inspect = Inspect.Unit.Detail
	local battle = SuperMeter.Battle
	local combat = SuperMeter_combat
	local forced = SuperMeter_forced
	local player = SuperMeter.Player
	local now = Inspect.Time.Real()
	local range = SuperMeter.Range
	local unify = SuperMeter_unify
	local sync = SuperMeter.Sync

	local unitName = false
	local unitID = false
	local owner = false
	local unit = false
	local run = false

	if not params.casterName or not params.targetName then
		if range[params.caster] and sync[params.caster] or range[params.target] and sync[params.target] then
			return unit
		end
	end

	if not combat then
		local time = session.startCombat + session.combatTime
		if now - time >= 0.88 or now - time < 0 then
			return unit
		end
	end

	if entity == "caster" then
		unitName = params.casterName
		unitID = params.caster
	end

	if entity == "target" then
		unitName = params.targetName
		unitID = params.target
	end

	if session.units[unitID] then
		return session.units[unitID]
	end

	if now then
		if not instance and not params.caster then return end
		if not instance and not params.target then return end
		
		if unitID then
			owner = inspect(string.format("%0s.owner", unitID))
			unit = inspect(unitID)
		end
		
		if not unit then
			unit = {id = (unitID or "Unknown"), name = unitName}
		end
		
		if owner and session.units[owner.id] then
			session.units[owner.id].pet = unitID
			return session.units[owner.id]
		end
		
		if unit and session.units[unit.id] then
			return session.units[unit.id]
		end
		
		if not unit.relation then
			unit.relation = "hostile"
		end
		
		if unit.nameSecondary then
			local name = string.match(unit.nameSecondary, "%a+")
			
			for k,v in pairs(session.units) do
				if unitID and session.units[k].name == name then
					session.units[k].pet = unitID
					return session.units[k]
				end
			end
		end
		
		for k,v in pairs(session.units) do
			if not owner and session.units[k].pet == unit.id then
				return session.units[k]
			end
			if owner and session.units[k].name == owner.name then
				return session.units[k]
			end
			if unify and session.units[k].name == unit.name then
				return session.units[k]
			end
		end
		
		if owner then unit = owner end
		
		if not unit.blocked and unit.combat or unit.relation == "hostile" then
			run = true
		end
		
		if battle[unit.id] or forced or instance or player.id == unit.id then
			run = true
		end
		
		if unit.name == "" or unit.name == " " or unit.name == "  " then
			unit.name = nil
		end
		
		if not run then return end
		
		local data = {damage = 0, damageAbsorbed = 0, damageAvoided = 0, damageBlocked = 0, damageDeflected = 0, damageMissed = 0, damageTaken = 0, healing = 0, healingTaken = 0, overHealing = 0, overKill = 0}
		
		if entity then
			data.casters = {damage = {}, damageAbsorbed = {}, damageBlocked = {}, damageDeflected = {}, healing = {}, hitless = {}, overHealing = {}, overKill = {}}
			data.targets = {damage = {}, damageAbsorbed = {}, damageBlocked = {}, damageDeflected = {}, healing = {}, hitless = {}, overHealing = {}, overKill = {}}
		end
		
		if unit then
			data.relation = unit.relation
			data.calling = unit.calling
			data.player = unit.player
			data.level = unit.level
			data.identity = unit.id
			data.name = unit.name
			data.abilities = {}
			data.combatlog = {}
		end
		
		if owner then
			data.pet = unitID
		end
		
		if not session.units[unit.id] then
			session.units[unit.id] = data
		end
		
		if unit then
			return session.units[unit.id]
		end
	end
end