------------------------------------------------------------------
-------------------- Super Meter - Extra Data --------------------
------------------------------------------------------------------

function SuperMeter.Absorb(params, target)
	if not SuperMeter_enabled then return end

	local buffs = Inspect.Buff.List(params.target)
	local session = SuperMeter.Session(true)
	local absorb = params.damageAbsorbed
	local filters = SuperMeter.Filters()
	local inspect = Inspect.Buff.Detail
	local ignore = SuperMeter.Ignore

	if not ignore[params.target] then
		ignore[params.target] = {}
	end

	local exile = ignore[params.target]

	local caster = false
	local data = false

	if absorb < 0 or not buffs or not exile then return end

	for k,v in pairs(buffs) do
		if not caster and not exile[k] then
			local buff = inspect(params.target, k)
			if buff and buff.caster and filters.absorbs[buff.name] then
				data = {ability = k, abilityName = buff.name, caster = buff.caster, target = params.target, targetName = target.name, heal = absorb}
				caster = session.units[buff.caster]
			end
			if not buff or not filters.absorbs[buff.name] then
				exile[k] = true
			end
		end
	end

	if data and caster then
		data.casterName = (caster and caster.player and caster.name)
		SuperMeter.Gather(data, "absorb", caster, target)
	end
end

function SuperMeter.Count(casters, targets)
	if not SuperMeter_enabled then return end

	local font = SuperMeter_fontSize
	local format = string.format

	local key = false
	local units = {}
	local total = 0
	local text = ""

	if casters then
		for k,v in pairs(casters) do
			table.insert(units, k)
			total = total + v
			key = casters
		end
	end

	if targets then
		for k,v in pairs(targets) do
			table.insert(units, k)
			total = total + v
			key = targets
		end
	end

	if key and #units > 0 then
		table.sort(units, function(a, b) return key[a] > key[b] end)
	end

	if #units > 8 then
		for i = 8 + 1, #units do
			units[i] = nil
		end
	end

	for k,v in pairs(units) do
		if font and font == 14 then
			text = format("%0s%0s", text, format("%-014s\t%-014s\t%-04s\n", format("[ %0s ]", key[v]), format("[ %.2f%% ]", (key[v] / total) * 100), v))
		else
			text = format("%0s%0s", text, format("%-016s\t%-016s\t%-04s\n", format("[ %0s ]", key[v]), format("[ %.2f%% ]", (key[v] / total) * 100), v))
		end
	end

	if #units > 0 and total > 0 and text then
		return format("\n%0s", text)
	end

	return text
end

function SuperMeter.Debug(input1, input2)
	if not SuperMeter_enabled then return end
	if not SuperMeter_debug then return end

	dump(input1, input2)
end

function SuperMeter.Full(params, full)
	if not SuperMeter_enabled then return end

	if params.damageIntercepted then
		full = full + params.damageIntercepted
	end

	if params.damageAbsorbed then
		full = full + params.damageAbsorbed
	end

	if params.damageDeflected then
		full = full + params.damageDeflected
	end

	if params.damageBlocked then
		full = full + params.damageBlocked
	end

	if params.overkill then
		full = full + params.overkill
	end

	if params.overheal then
		full = full + params.overheal
	end

	if params.damage then
		full = full + params.damage
	end

	if params.heal then
		full = full + params.heal
	end

	return full
end

function SuperMeter.Merge()
	if not SuperMeter_enabled then return end

	local merge = {allies = {}, combatTime = 0, deathlog = {}, enemies = {}, merge = true, startCombat = 0, units = {}}
	local interface = SuperMeter.Interface
	local sessions = SuperMeter.Sessions
	local integer = tonumber

	for k1,v1 in pairs(sessions) do
		if merge.combatTime and v1.combatTime then
			merge.combatTime = merge.combatTime + v1.combatTime
		end
		
		if merge.startCombat and v1.startCombat then
			merge.startCombat = merge.startCombat + v1.startCombat
		end
		
		for k2,v2 in pairs(v1.units) do
			local current = merge.units[k2]
			
			if not current then
				merge.units[k2] = {}
				
				for k3,v3 in pairs(v2) do
					merge.units[k2][k3] = v3
				end
			end
			
			if not current then
				merge.units[k2].abilities = {}
				merge.units[k2].combatlog = {}
				merge.units[k2].casters = {}
				merge.units[k2].targets = {}
			end
			
			for k3,v3 in pairs(v2) do
				if current and current[k3] and integer(v3) then
					current[k3] = current[k3] + v3
					current.level = v2.level
				end
			end
		end
	end

	if #sessions >= 2 then
		interface.Manager(false, false)
		table.insert(sessions, merge)
	end

	interface.Update()
end

function SuperMeter.Upgrade()
	local export = SuperMeter.Export

	for k1,v1 in pairs(export) do
		for k2,v2 in pairs(v1.units) do
			if v2 and not v2.identity then
				v2.identity = k2
			end
			if v2 and not v2.casters then
				v2.casters = {}
			end
			if v2 and not v2.targets then
				v2.targets = {}
			end
		end
	end
end